#!/usr/bin/env python2.7

import platform, sys, os, math
import locale
import getopt
import numpy as np
import cv2

from PIL import Image

from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *

global cap
global frame
global size
global out
global output
global texName
global vidpause
global faces
global eyes
global smiles
global mouths
global noses

def usage(argv):
  print "Usage: ", argv[0], "\n" \
        "  --output\n" \
        "    output: if present output video\n"

def reshape(width, height):
  glViewport(0, 0, width, height)

  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()

  # l, r, b, t, near, far
# glOrtho(-width/height, width/height, -1.0, 1.0, -1.0, 1.0)
  glOrtho(0, width, 0, height, -1.0, 1.0)
# gluOrtho2D(0, width, 0, height)

  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()

def keyboard(key, x, y):

  global vidpause

  if key is '\033':
    cleanup()
  elif key is 'q':
    cleanup()
  elif key == 'p' or key == ' ':
    vidpause = 1 - vidpause
  else:
    print "unknown key"

  glutPostRedisplay()

def init():

  global texName

  glEnable(GL_DEPTH_TEST)

  glClearColor(0.22, 0.28, 0.34, 0.0)

  glShadeModel(GL_FLAT)

  # obtain texture id from OpenGL
  texName = glGenTextures(1)
# print "got texName = ", texName

  # blending function
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  glEnable(GL_BLEND)

  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

def texbind(frame):

  # convert OpenCV cv2 frame to OpenGL texture format
# gray = cv2.cvtColor(frame,cv2.COLOR_BGR2GRAY)
  frame = cv2.flip(frame,0)
  frame = cv2.cvtColor(frame,cv2.COLOR_BGR2RGB)
  img = Image.fromarray(frame)

  # img should be compatible with PIL Image
  (imw, imh) = img.size

  # which texture we are manipulating
  glBindTexture(GL_TEXTURE_2D,texName)

  # set up texture parameters
# glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP)
# glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP)
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT)
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT)
# glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST)
# glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST)
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR)
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR)

  # transfer image pixels to texture memory
  # target, level, internalFormat, img.w, img.h, border, format, type, pixels
  glTexImage2D(GL_TEXTURE_2D,
               0,
               GL_RGBA,
               imw,imh,
               0,
               GL_RGBA,
               GL_UNSIGNED_BYTE,
               img.convert("RGBA").tobytes())

  # GL_DECAL or GL_REPLACE will mask lighting, use GL_MODULATE instead
# glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL)
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)
# glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)

def display():

  global out
  global output
  global faces
  global eyes
  global smiles
  global mouths
  global noses

  width = glutGet(GLUT_WINDOW_WIDTH)
  height = glutGet(GLUT_WINDOW_HEIGHT)

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  glOrtho(0, width, 0, height, -1.0, 1.0)

  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()

  # text color
  glColor4f(1.0, 1.0, 1.0, 1.0)

  # draw faces
  if faces is not None:
    for (fx,fy,fw,fh) in faces:
#     print x,y,w,h
      x1 = fx
      y1 = fy
      x2 = x1 + fw
      y2 = y1 + fh
      y1 = height - y1
      y2 = height - y2

      glBegin(GL_LINE_LOOP)
      glVertex2f(x1,y1)
      glVertex2f(x2,y1)
      glVertex2f(x2,y2)
      glVertex2f(x1,y2)
      glEnd()

      # draw eyes
      glColor4f(1.0, 0.0, 0.0, 1.0)
      if eyes is not None:
        for (ex,ey,ew,eh) in eyes:
#         print x,y,w,h
          x1 = ex + fx
          y1 = ey + fy
          x2 = x1 + ew
          y2 = y1 + eh
          y1 = height - y1
          y2 = height - y2

          glBegin(GL_LINE_LOOP)
          glVertex2f(x1,y1)
          glVertex2f(x2,y1)
          glVertex2f(x2,y2)
          glVertex2f(x1,y2)
          glEnd()

      # draw smiles
#     glColor4f(0.0, 1.0, 0.0, 1.0)
#     if smiles is not None:
#       for (sx,sy,sw,sh) in smiles:
##        print x,y,w,h
#         x1 = sx + fx
#         y1 = sy + fy
#         x2 = x1 + sw
#         y2 = y1 + sh
#         y1 = height - y1
#         y2 = height - y2

#         glBegin(GL_LINE_LOOP)
#         glVertex2f(x1,y1)
#         glVertex2f(x2,y1)
#         glVertex2f(x2,y2)
#         glVertex2f(x1,y2)
#         glEnd()

      # draw mouths
      glColor4f(0.0, 1.0, 0.0, 1.0)
      if mouths is not None:
        for (mx,my,mw,mh) in mouths:
#         print x,y,w,h
          x1 = mx + fx
          y1 = my + fy
          x2 = x1 + mw
          y2 = y1 + mh
          y1 = height - y1
          y2 = height - y2

          glBegin(GL_LINE_LOOP)
          glVertex2f(x1,y1)
          glVertex2f(x2,y1)
          glVertex2f(x2,y2)
          glVertex2f(x1,y2)
          glEnd()

      # draw noses
      glColor4f(0.0, 0.0, 1.0, 1.0)
      if noses is not None:
        for (nx,ny,nw,nh) in noses:
#         print x,y,w,h
          x1 = nx + fx
          y1 = ny + fy
          x2 = x1 + nw
          y2 = y1 + nh
          y1 = height - y1
          y2 = height - y2

          glBegin(GL_LINE_LOOP)
          glVertex2f(x1,y1)
          glVertex2f(x2,y1)
          glVertex2f(x2,y2)
          glVertex2f(x1,y2)
          glEnd()

  # draw texture map (video frame)
  glEnable(GL_TEXTURE_2D)
  glBegin(GL_QUADS)

  glTexCoord2f(0.0, 0.0)
  glVertex3f(0.0, 0.0, 0.0)

  glTexCoord2f(1.0, 0.0)
  glVertex3f(width, 0.0, 0.0)

  glTexCoord2f(1.0, 1.0)
  glVertex3f(width, height, 0.0)

  glTexCoord2f(0.0, 1.0)
  glVertex3f(0.0, height, 0.0)
  glEnd()
  glDisable(GL_TEXTURE_2D)

  glFlush()
  glutSwapBuffers()

  # write out the frame, here we would need to screen grab
  if output:
    glReadBuffer(GL_FRONT)
    buf = glReadPixels(0,0,width,height,GL_RGBA,GL_UNSIGNED_BYTE)
    img = Image.frombytes(mode="RGBA",size=(width,height),data=buf)
#   img = img.transpose(Image.FLIP_TOP_BOTTOM)
    # convert OpenGL texture format to OpenCV cv2 frame
#   gray = cv2.cvtColor(np.asarray(img),cv2.COLOR_BGR2GRAY)
    frame = cv2.cvtColor(np.asarray(img),cv2.COLOR_RGB2BGR)
    frame = cv2.flip(frame,0)

    out.write(frame)

def main(argv):

  global cap
  global frame
  global size
  global texName
  global out
  global output
  global faces
  global eyes
  global smiles
  global mouths
  global noses

  cap = None
  texName = None
  output = False
  faces = None
  eyes = None
  smiles = None
  mouths = None
  noses = None

  # get args
  try:
    opts, args = getopt.getopt(argv[1:], '', \
                 ['output'])
  except getopt.GetoptError, err:
    print "getopt error: ", err
    usage(argv)
    cleanup()

  # process args
  for opt,arg in opts:
    if opt == '--output':
      output = True
    else:
      sys.argv[1:]

  if output:
    print "Outputting to 'output.mov'"

  # gram video from available camera (e.g., laptop cam)
  cap = cv2.VideoCapture(1)

  size = (int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)), \
          int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)))

  fps = cap.get(cv2.CAP_PROP_FPS)

  # resize video by (.5,.5)
# size = (size[0]/4, size[1]/4)
# print "size, fps: ", size, fps
# cap.set(size[0],size[1])

 # for output
  if output:
    fourcc = cv2.VideoWriter_fourcc(*'mp4v') # MPEG not supported?
#   fps = 25.0
    out = cv2.VideoWriter('output.mov',fourcc,fps,size,True)

  # init glut
  glutInit(sys.argv)
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH)
  glutInitWindowPosition(500,500)
  window = glutCreateWindow('Python GL Video -- Esc to exit')
  glutSetWindow(window)
  #glutFullScreen()
  glutReshapeWindow(size[0],size[1])

  # set up callbacks
  glutReshapeFunc(reshape)
  glutDisplayFunc(display)
  glutIdleFunc(idle)
  glutKeyboardFunc(keyboard)
# glutMotionFunc(motion)
# glutMouseFunc(mouse)

  init()
  glutMainLoop()

def cleanup():

  # when everything is done, release the capture
  if cap is not None:
    cap.release()

  if output:
    out.release()

  #if using cv2 to play video
# cv2.destroyAllWindows()

  sys.exit()

def idle():

  global cap
  global frame
  global size
  global output
  global out

  if cap.isOpened():

    # capture frame-by-frame
    ret, frame = cap.read()

    if ret is True:

      frame = cv2.resize(frame, size, 0, 0, cv2.INTER_CUBIC)

      # our operations on the frame come here
      gray = cv2.cvtColor(frame,cv2.COLOR_BGR2GRAY)

      # display the resulting frame
#     cv2.imshow('frame',gray)
#     if cv2.waitKey(1) & 0xFF == ord('q'):
#       break

      detectFaces(gray)

      texbind(frame)

      glutPostRedisplay()

def detectFaces(gray):

  global faces
  global eyes
  global smiles
  global mouths
  global noses

  (r,c) = gray.shape
# print "h x w: ", r, c

# faceCascadeName = './haarcascades/haarcascade_frontalface_default.xml'
# faceCascadeName = './haarcascades/haarcascade_frontalface_alt.xml'
  faceCascadeName = './haarcascades/haarcascade_frontalface_alt2.xml'
# faceCascadeName = './haarcascades/haarcascade_frontalface_alt_tree.xml'

# eyeCascadeName = './haarcascades/haarcascade_eye.xml'
# eyeCascadeName = './haarcascades/haarcascade_eye_tree_eyeglasses.xml'
# eyeCascadeName = './haarcascades/haarcascade_lefteye_2splits.xml'
  eyeCascadeName = './haarcascades/haarcascade_righteye_2splits.xml'

  smileCascadeName = './haarcascades/haarcascade_smile.xml'

  noseCascadeName = './haarcascades/Nariz.xml'

  mouthCascadeName = './haarcascades/Mouth.xml'

  face_cascade = cv2.CascadeClassifier(faceCascadeName)
  eye_cascade = cv2.CascadeClassifier(eyeCascadeName)
  smile_cascade = cv2.CascadeClassifier(smileCascadeName)
  nose_cascade = cv2.CascadeClassifier(noseCascadeName)
  mouth_cascade = cv2.CascadeClassifier(mouthCascadeName)

  faces = face_cascade.detectMultiScale(gray, 1.3, 5)
# print "len(faces): ", len(faces)

  for (x,y,w,h) in faces:

    # use subimage for search within face
    roi_gray = gray[y:y+h, x:x+w]

    # look for eyes
    eyes = eye_cascade.detectMultiScale(roi_gray)
#    for i in xrange(len(eyes)):
##     print "eyes[", i, "]: ", eyes[i]
#      eyes[i][0] += x
#      eyes[i][1] += y

    # look for smiles
    smiles = smile_cascade.detectMultiScale(roi_gray)

    # look for noses
    noses = nose_cascade.detectMultiScale(roi_gray)

    # look for mouths
    mouths = mouth_cascade.detectMultiScale(roi_gray)

if __name__ == '__main__':
# main(sys.argv[1:])
  main(sys.argv)
