#!/usr/bin/env python2

import sys,os,getopt,glob
import locale
import numpy as np

def usage():
  print "Usage: python tsv2raw.py " \
        " --indir=? -outdir=?\n" \
        " --file=?\n" \
        "   indir: a directory containing input files to be processed\n" \
        "   outdir: a directory containing output files\n" \
        "   file: a single file to process\n"

# convert csv file to raw
def tsv2raw(infile,outdir):
  try:
    f = open(infile,'r')
  except IOError:
    print "Can't open file: " + infile
    return

  infile = infile.replace('\\','/')
	
  # parse the subject name from the file name
# base = infile[:infile.rfind('.')]
  base = os.path.basename(infile)
  print "Processing: ", infile, "[", base, "]"

  # strip out extension
  filename, ext = os.path.splitext(base)
  print 'Processing: ' + filename
  
  subj = infile.split('/')[-3]
  trial = infile.split('/')[-2]
  
# print 'subj, trial: ', subj, trial
# print 'Processing: %s [%s]' % (subj,trial)

  outfile = None
  outfile = open(outdir + subj + '_' + trial + '.raw','w+')

  # read lines, throwing away first one (header)
  linelist = f.read().splitlines()
  header = linelist[0].split('\t')
  linelist = linelist[1:]

  for idx, label in enumerate(header):
    if "sync.timestamp" in label.strip():
      SYNCTIME = idx
    if "field.timestamp" in label.strip():
      FIELDTIME = idx
    if label.strip() == "field.gaze.x":
      BPOGX = idx
    if label.strip() == "field.gaze.y":
      BPOGY = idx
    if label.strip() == "field.width":
      FWIDTH = idx
    if label.strip() == "field.height":
      FHEIGHT = idx
    if label.strip() == "field.gaze.valid":
      VALID = idx

# print "header::", header
# print "len(header)::", len(header)
# exit()

  # reset coords
  x = ''
  y = ''
  t = ''

  # process each line, splitting on ','
  for line in linelist:
    elements = line.split('\t')
    entry = np.asarray(elements)

    # don't process corrupt lines
    if len(elements) < len(header):
      continue

    # dump out gaze coordinates
    if(outfile is not None and
       entry[BPOGX] != x and
       entry[BPOGY] != y and
       entry[SYNCTIME] != t):

       # data already normalized, no need to divide by screen size
       fw = float(entry[FWIDTH])
       fh = float(entry[FHEIGHT])
       if fw != 0.0 and fh != 0.0:
         x = str(float(entry[BPOGX])/fw)
         y = str(float(entry[BPOGY])/fh)
       else:
         x = '0'
         y = '0'
       t = entry[SYNCTIME]

       # don't process empty or negative coords
#      if locale.atof(x) < 0 or locale.atof(y) < 0:
#        print "locale data: ",locale.atof(x),locale.atof(y),"*******"
#      else:
#        print "locale data: ",locale.atof(x),locale.atof(y)
       if(x != '' and y != '' and \
          locale.atof(x) < 1 and locale.atof(y) < 1 and \
          locale.atof(x) >= 0 and locale.atof(y) >= 0):
         strout = "%f %f %f" % (locale.atof(x),locale.atof(y),float(t))
         outfile.write(strout + '\n')

def main(argv):
# if not len(argv):
#   usage()

  try:
    opts, args = getopt.getopt(argv, '', \
                 ['indir=','outdir=','file=',\
                  'hertz=','sfdegree=','sfcutoff='])
  except getopt.GetoptError, err:
    usage()
    exit()

  file = None
  files = []

  for opt,arg in opts:
    opt = opt.lower()
    if(opt != '--file'):
      arg = arg.lower()

    if opt == '--indir':
      indir = arg
    elif opt == '--outdir':
      outdir = arg
    else:
      sys.argv[1:]

  # get .csv input files to process
  if os.path.isdir(indir):
#   files = glob.glob('%s/*.tsv' % (indir))
#   files = glob.glob('%s/JournalData.tsv' % (indir))
    for top, dirs, listing in os.walk(indir):
      for file in listing:
        path = os.path.join(top,file)
        dname = path.split('/')[2]
        base = os.path.basename(path)
        if base.endswith('JournalData.tsv'):
          basename = base[:base.rfind('.tsv')]
#         print path, top, dname, base, "[",basename,"]","[",file,"]"
          files.extend([path])
        if base.endswith('.txt'):
          basename = base[:base.rfind('.txt')]
#         print path, top, dname, base, "[",basename,"]","[",file,"]"
          files.extend([path])

  print "files: ", files

  # if user specified --file="..." then we use that as the only one to process
  if(file != None and os.path.isfile(file)):
    files = [file]

  for file in files:
    tsv2raw(file,outdir)

#######################
#locale.setlocale( locale.LC_ALL, 'en_US.UTF-8' )

if __name__ == "__main__":
  main(sys.argv[1:])
